@@ -5,11 +5,12 @@ use std::io::SeekFrom;
 use std::path::{Path, PathBuf};
 use std::rc::Rc;
 use std::sync::Arc;
+use std::task::Poll;
 
 use crate::core::compiler::{BuildConfig, CompileMode, DefaultExecutor, Executor};
 use crate::core::resolver::CliFeatures;
 use crate::core::{Feature, Shell, Verbosity, Workspace};
-use crate::core::{Package, PackageId, PackageSet, Resolve, Source, SourceId};
+use crate::core::{Package, PackageId, PackageSet, Resolve, SourceId};
 use crate::sources::PathSource;
 use crate::util::errors::CargoResult;
 use crate::util::toml::TomlManifest;
@@ -20,6 +21,7 @@ use cargo_util::paths;
 use flate2::read::GzDecoder;
 use flate2::{Compression, GzBuilder};
 use log::debug;
+use serde::Serialize;
 use tar::{Archive, Builder, EntryType, Header, HeaderMode};
 
 pub struct PackageOpts<'cfg> {
@@ -28,12 +30,14 @@ pub struct PackageOpts<'cfg> {
     pub check_metadata: bool,
     pub allow_dirty: bool,
     pub verify: bool,
-    pub jobs: Option<u32>,
+    pub jobs: Option<i32>,
+    pub keep_going: bool,
     pub to_package: ops::Packages,
     pub targets: Vec<String>,
     pub cli_features: CliFeatures,
 }
 
+const ORIGINAL_MANIFEST_FILE: &str = "Cargo.toml.orig";
 const VCS_INFO_FILE: &str = ".cargo_vcs_info.json";
 
 struct ArchiveFile {
@@ -58,8 +62,20 @@ enum GeneratedFile {
     Manifest,
     /// Generates `Cargo.lock` in some cases (like if there is a binary).
     Lockfile,
-    /// Adds a `.cargo-vcs_info.json` file if in a (clean) git repo.
-    VcsInfo(String),
+    /// Adds a `.cargo_vcs_info.json` file if in a (clean) git repo.
+    VcsInfo(VcsInfo),
+}
+
+#[derive(Serialize)]
+struct VcsInfo {
+    git: GitVcsInfo,
+    /// Path to the package within repo (empty string if root). / not \
+    path_in_vcs: String,
+}
+
+#[derive(Serialize)]
+struct GitVcsInfo {
+    sha1: String,
 }
 
 pub fn package_one(
@@ -88,7 +104,6 @@ pub fn package_one(
     let vcs_info = if !opts.allow_dirty {
         // This will error if a dirty repo is found.
         check_repo_state(pkg, &src_files, config)?
-            .map(|h| format!("{{\n  \"git\": {{\n    \"sha1\": \"{}\"\n  }}\n}}\n", h))
     } else {
         None
     };
@@ -165,6 +180,7 @@ pub fn package(ws: &Workspace<'_>, opts: &PackageOpts<'_>) -> CargoResult<Option
                 allow_dirty: opts.allow_dirty,
                 verify: opts.verify,
                 jobs: opts.jobs,
+                keep_going: opts.keep_going,
                 to_package: ops::Packages::Default,
                 targets: opts.targets.clone(),
                 cli_features: cli_features,
@@ -189,7 +205,7 @@ fn build_ar_list(
     ws: &Workspace<'_>,
     pkg: &Package,
     src_files: Vec<PathBuf>,
-    vcs_info: Option<String>,
+    vcs_info: Option<VcsInfo>,
 ) -> CargoResult<Vec<ArchiveFile>> {
     let mut result = Vec::new();
     let root = pkg.root();
@@ -205,8 +221,8 @@ fn build_ar_list(
         match rel_str.as_ref() {
             "Cargo.toml" => {
                 result.push(ArchiveFile {
-                    rel_path: PathBuf::from("Cargo.toml.orig"),
-                    rel_str: "Cargo.toml.orig".to_string(),
+                    rel_path: PathBuf::from(ORIGINAL_MANIFEST_FILE),
+                    rel_str: ORIGINAL_MANIFEST_FILE.to_string(),
                     contents: FileContents::OnDisk(src_file),
                 });
                 result.push(ArchiveFile {
@@ -216,10 +232,9 @@ fn build_ar_list(
                 });
             }
             "Cargo.lock" => continue,
-            VCS_INFO_FILE => anyhow::bail!(
-                "invalid inclusion of reserved file name \
-                     {} in package source",
-                VCS_INFO_FILE
+            VCS_INFO_FILE | ORIGINAL_MANIFEST_FILE => anyhow::bail!(
+                "invalid inclusion of reserved file name {} in package source",
+                rel_str
             ),
             _ => {
                 result.push(ArchiveFile {
@@ -246,46 +261,16 @@ fn build_ar_list(
     }
     if let Some(license_file) = &pkg.manifest().metadata().license_file {
         let license_path = Path::new(license_file);
-        let abs_license_path = paths::normalize_path(&pkg.root().join(license_path));
-        if abs_license_path.exists() {
-            match abs_license_path.strip_prefix(&pkg.root()) {
-                Ok(rel_license_path) => {
-                    if !result.iter().any(|ar| ar.rel_path == rel_license_path) {
-                        result.push(ArchiveFile {
-                            rel_path: rel_license_path.to_path_buf(),
-                            rel_str: rel_license_path
-                                .to_str()
-                                .expect("everything was utf8")
-                                .to_string(),
-                            contents: FileContents::OnDisk(abs_license_path),
-                        });
-                    }
-                }
-                Err(_) => {
-                    // The license exists somewhere outside of the package.
-                    let license_name = license_path.file_name().unwrap();
-                    if result
-                        .iter()
-                        .any(|ar| ar.rel_path.file_name().unwrap() == license_name)
-                    {
-                        ws.config().shell().warn(&format!(
-                            "license-file `{}` appears to be a path outside of the package, \
-                            but there is already a file named `{}` in the root of the package. \
-                            The archived crate will contain the copy in the root of the package. \
-                            Update the license-file to point to the path relative \
-                            to the root of the package to remove this warning.",
-                            license_file,
-                            license_name.to_str().unwrap()
-                        ))?;
-                    } else {
-                        result.push(ArchiveFile {
-                            rel_path: PathBuf::from(license_name),
-                            rel_str: license_name.to_str().unwrap().to_string(),
-                            contents: FileContents::OnDisk(abs_license_path),
-                        });
-                    }
-                }
-            }
+        let abs_file_path = paths::normalize_path(&pkg.root().join(license_path));
+        if abs_file_path.exists() {
+            check_for_file_and_add(
+                "license-file",
+                license_path,
+                abs_file_path,
+                pkg,
+                &mut result,
+                ws,
+            )?;
         } else {
             let rel_msg = if license_path.is_absolute() {
                 "".to_string()
@@ -302,11 +287,69 @@ fn build_ar_list(
             ))?;
         }
     }
+    if let Some(readme) = &pkg.manifest().metadata().readme {
+        let readme_path = Path::new(readme);
+        let abs_file_path = paths::normalize_path(&pkg.root().join(readme_path));
+        if abs_file_path.exists() {
+            check_for_file_and_add("readme", readme_path, abs_file_path, pkg, &mut result, ws)?;
+        }
+    }
     result.sort_unstable_by(|a, b| a.rel_path.cmp(&b.rel_path));
 
     Ok(result)
 }
 
+fn check_for_file_and_add(
+    label: &str,
+    file_path: &Path,
+    abs_file_path: PathBuf,
+    pkg: &Package,
+    result: &mut Vec<ArchiveFile>,
+    ws: &Workspace<'_>,
+) -> CargoResult<()> {
+    match abs_file_path.strip_prefix(&pkg.root()) {
+        Ok(rel_file_path) => {
+            if !result.iter().any(|ar| ar.rel_path == rel_file_path) {
+                result.push(ArchiveFile {
+                    rel_path: rel_file_path.to_path_buf(),
+                    rel_str: rel_file_path
+                        .to_str()
+                        .expect("everything was utf8")
+                        .to_string(),
+                    contents: FileContents::OnDisk(abs_file_path),
+                })
+            }
+        }
+        Err(_) => {
+            // The file exists somewhere outside of the package.
+            let file_name = file_path.file_name().unwrap();
+            if result
+                .iter()
+                .any(|ar| ar.rel_path.file_name().unwrap() == file_name)
+            {
+                ws.config().shell().warn(&format!(
+                    "{} `{}` appears to be a path outside of the package, \
+                            but there is already a file named `{}` in the root of the package. \
+                            The archived crate will contain the copy in the root of the package. \
+                            Update the {} to point to the path relative \
+                            to the root of the package to remove this warning.",
+                    label,
+                    file_path.display(),
+                    file_name.to_str().unwrap(),
+                    label,
+                ))?;
+            } else {
+                result.push(ArchiveFile {
+                    rel_path: PathBuf::from(file_name),
+                    rel_str: file_name.to_str().unwrap().to_string(),
+                    contents: FileContents::OnDisk(abs_file_path),
+                })
+            }
+        }
+    }
+    Ok(())
+}
+
 /// Construct `Cargo.lock` for the package to be published.
 fn build_lock(ws: &Workspace<'_>, orig_pkg: &Package) -> CargoResult<String> {
     let config = ws.config();
@@ -332,7 +375,12 @@ fn build_lock(ws: &Workspace<'_>, orig_pkg: &Package) -> CargoResult<String> {
     if let Some(orig_resolve) = orig_resolve {
         compare_resolve(config, tmp_ws.current()?, &orig_resolve, &new_resolve)?;
     }
-    check_yanked(config, &pkg_set, &new_resolve)?;
+    check_yanked(
+        config,
+        &pkg_set,
+        &new_resolve,
+        "consider updating to a version that is not yanked",
+    )?;
 
     ops::resolve_to_string(&tmp_ws, &mut new_resolve)
 }
@@ -386,7 +434,7 @@ fn check_repo_state(
     p: &Package,
     src_files: &[PathBuf],
     config: &Config,
-) -> CargoResult<Option<String>> {
+) -> CargoResult<Option<VcsInfo>> {
     if let Ok(repo) = git2::Repository::discover(p.root()) {
         if let Some(workdir) = repo.workdir() {
             debug!("found a git repo at {:?}", workdir);
@@ -398,7 +446,15 @@ fn check_repo_state(
                         "found (git) Cargo.toml at {:?} in workdir {:?}",
                         path, workdir
                     );
-                    return git(p, src_files, &repo);
+                    let path_in_vcs = path
+                        .parent()
+                        .and_then(|p| p.to_str())
+                        .unwrap_or("")
+                        .replace("\\", "/");
+                    return Ok(Some(VcsInfo {
+                        git: git(p, src_files, &repo)?,
+                        path_in_vcs,
+                    }));
                 }
             }
             config.shell().verbose(|shell| {
@@ -419,11 +475,7 @@ fn check_repo_state(
     // directory is dirty or not, thus we have to assume that it's clean.
     return Ok(None);
 
-    fn git(
-        p: &Package,
-        src_files: &[PathBuf],
-        repo: &git2::Repository,
-    ) -> CargoResult<Option<String>> {
+    fn git(p: &Package, src_files: &[PathBuf], repo: &git2::Repository) -> CargoResult<GitVcsInfo> {
         // This is a collection of any dirty or untracked files. This covers:
         // - new/modified/deleted/renamed/type change (index or worktree)
         // - untracked files (which are "new" worktree files)
@@ -450,7 +502,9 @@ fn check_repo_state(
             .collect();
         if dirty_src_files.is_empty() {
             let rev_obj = repo.revparse_single("HEAD")?;
-            Ok(Some(rev_obj.id().to_string()))
+            Ok(GitVcsInfo {
+                sha1: rev_obj.id().to_string(),
+            })
         } else {
             anyhow::bail!(
                 "{} files in the working directory contain changes that were \
@@ -562,7 +616,7 @@ fn tar(
                 let contents = match generated_kind {
                     GeneratedFile::Manifest => pkg.to_registry_toml(ws)?,
                     GeneratedFile::Lockfile => build_lock(ws, pkg)?,
-                    GeneratedFile::VcsInfo(s) => s,
+                    GeneratedFile::VcsInfo(ref s) => serde_json::to_string_pretty(s)?,
                 };
                 header.set_entry_type(EntryType::file());
                 header.set_mode(0o644);
@@ -668,22 +722,45 @@ fn compare_resolve(
     Ok(())
 }
 
-fn check_yanked(config: &Config, pkg_set: &PackageSet<'_>, resolve: &Resolve) -> CargoResult<()> {
+pub fn check_yanked(
+    config: &Config,
+    pkg_set: &PackageSet<'_>,
+    resolve: &Resolve,
+    hint: &str,
+) -> CargoResult<()> {
     // Checking the yanked status involves taking a look at the registry and
     // maybe updating files, so be sure to lock it here.
     let _lock = config.acquire_package_cache_lock()?;
 
     let mut sources = pkg_set.sources_mut();
-    for pkg_id in resolve.iter() {
-        if let Some(source) = sources.get_mut(pkg_id.source_id()) {
-            if source.is_yanked(pkg_id)? {
-                config.shell().warn(format!(
-                    "package `{}` in Cargo.lock is yanked in registry `{}`, \
-                     consider updating to a version that is not yanked",
-                    pkg_id,
-                    pkg_id.source_id().display_registry_name()
-                ))?;
+    let mut pending: Vec<PackageId> = resolve.iter().collect();
+    let mut results = Vec::new();
+    for (_id, source) in sources.sources_mut() {
+        source.invalidate_cache();
+    }
+    while !pending.is_empty() {
+        pending.retain(|pkg_id| {
+            if let Some(source) = sources.get_mut(pkg_id.source_id()) {
+                match source.is_yanked(*pkg_id) {
+                    Poll::Ready(result) => results.push((*pkg_id, result)),
+                    Poll::Pending => return true,
+                }
             }
+            false
+        });
+        for (_id, source) in sources.sources_mut() {
+            source.block_until_ready()?;
+        }
+    }
+
+    for (pkg_id, is_yanked) in results {
+        if is_yanked? {
+            config.shell().warn(format!(
+                "package `{}` in Cargo.lock is yanked in registry `{}`, {}",
+                pkg_id,
+                pkg_id.source_id().display_registry_name(),
+                hint
+            ))?;
         }
     }
     Ok(())
@@ -737,7 +814,13 @@ fn run_verify(
     ops::compile_with_exec(
         &ws,
         &ops::CompileOptions {
-            build_config: BuildConfig::new(config, opts.jobs, &opts.targets, CompileMode::Build)?,
+            build_config: BuildConfig::new(
+                config,
+                opts.jobs,
+                opts.keep_going,
+                &opts.targets,
+                CompileMode::Build,
+            )?,
             cli_features: opts.cli_features.clone(),
             spec: ops::Packages::Packages(Vec::new()),
             filter: ops::CompileFilter::Default {
@@ -745,6 +828,7 @@ fn run_verify(
             },
             target_rustdoc_args: None,
             target_rustc_args: rustc_args,
+            target_rustc_crate_types: None,
             local_rustdoc_args: None,
             rustdoc_document_private_items: false,
             honor_rust_version: true,
