@@ -3,17 +3,19 @@ use std::env;
 use std::io::prelude::*;
 use std::io::SeekFrom;
 use std::path::{Path, PathBuf};
+use std::rc::Rc;
+use std::task::Poll;
 
-use anyhow::{bail, format_err};
-use semver::VersionReq;
+use anyhow::{bail, format_err, Context as _};
 use serde::{Deserialize, Serialize};
+use toml_edit::easy as toml;
 
 use crate::core::compiler::Freshness;
-use crate::core::{Dependency, Package, PackageId, Source, SourceId};
+use crate::core::{Dependency, FeatureValue, Package, PackageId, QueryKind, Source, SourceId};
 use crate::ops::{self, CompileFilter, CompileOptions};
 use crate::sources::PathSource;
-use crate::util::errors::{CargoResult, CargoResultExt};
-use crate::util::{Config, ToSemver};
+use crate::util::errors::CargoResult;
+use crate::util::Config;
 use crate::util::{FileLock, Filesystem};
 
 /// On-disk tracking for which package installed which binary.
@@ -101,12 +103,11 @@ impl InstallTracker {
             if contents.is_empty() {
                 Ok(CrateListingV1::default())
             } else {
-                Ok(toml::from_str(&contents)
-                    .chain_err(|| format_err!("invalid TOML found for metadata"))?)
+                Ok(toml::from_str(&contents).with_context(|| "invalid TOML found for metadata")?)
             }
         })()
-        .chain_err(|| {
-            format_err!(
+        .with_context(|| {
+            format!(
                 "failed to parse crate metadata at `{}`",
                 v1_lock.path().to_string_lossy()
             )
@@ -119,13 +120,13 @@ impl InstallTracker {
                 CrateListingV2::default()
             } else {
                 serde_json::from_str(&contents)
-                    .chain_err(|| format_err!("invalid JSON found for metadata"))?
+                    .with_context(|| "invalid JSON found for metadata")?
             };
-            v2.sync_v1(&v1)?;
+            v2.sync_v1(&v1);
             Ok(v2)
         })()
-        .chain_err(|| {
-            format_err!(
+        .with_context(|| {
+            format!(
                 "failed to parse crate metadata at `{}`",
                 v2_lock.path().to_string_lossy()
             )
@@ -159,7 +160,7 @@ impl InstallTracker {
         dst: &Path,
         pkg: &Package,
         force: bool,
-        opts: &CompileOptions<'_>,
+        opts: &CompileOptions,
         target: &str,
         _rustc: &str,
     ) -> CargoResult<(Freshness, BTreeMap<String, Option<PackageId>>)> {
@@ -231,7 +232,7 @@ impl InstallTracker {
                 if let Some(p) = p.as_ref() {
                     msg.push_str(&format!(" as part of `{}`\n", p));
                 } else {
-                    msg.push_str("\n");
+                    msg.push('\n');
                 }
             }
             msg.push_str("Add --force to overwrite");
@@ -267,7 +268,7 @@ impl InstallTracker {
         package: &Package,
         bins: &BTreeSet<String>,
         version_req: Option<String>,
-        opts: &CompileOptions<'_>,
+        opts: &CompileOptions,
         target: &str,
         rustc: &str,
     ) {
@@ -278,15 +279,15 @@ impl InstallTracker {
 
     /// Save tracking information to disk.
     pub fn save(&self) -> CargoResult<()> {
-        self.v1.save(&self.v1_lock).chain_err(|| {
-            format_err!(
+        self.v1.save(&self.v1_lock).with_context(|| {
+            format!(
                 "failed to write crate metadata at `{}`",
                 self.v1_lock.path().to_string_lossy()
             )
         })?;
 
-        self.v2.save(&self.v2_lock).chain_err(|| {
-            format_err!(
+        self.v2.save(&self.v2_lock).with_context(|| {
+            format!(
                 "failed to write crate metadata at `{}`",
                 self.v2_lock.path().to_string_lossy()
             )
@@ -356,7 +357,7 @@ impl CrateListingV1 {
         let mut file = lock.file();
         file.seek(SeekFrom::Start(0))?;
         file.set_len(0)?;
-        let data = toml::to_string(self)?;
+        let data = toml::to_string_pretty(self)?;
         file.write_all(data.as_bytes())?;
         Ok(())
     }
@@ -368,7 +369,7 @@ impl CrateListingV2 {
     /// where v2 is in use, and a v1 update is made, then v2 is used again.
     /// i.e., `cargo +new install foo ; cargo +old install bar ; cargo +new install bar`
     /// For now, v1 is the source of truth, so its values are trusted over v2.
-    fn sync_v1(&mut self, v1: &CrateListingV1) -> CargoResult<()> {
+    fn sync_v1(&mut self, v1: &CrateListingV1) {
         // Make the `bins` entries the same.
         for (pkg_id, bins) in &v1.v1 {
             self.installs
@@ -386,7 +387,6 @@ impl CrateListingV2 {
         for pkg_id in to_remove {
             self.installs.remove(&pkg_id);
         }
-        Ok(())
     }
 
     fn package_for_bin(&self, bin_name: &str) -> Option<PackageId> {
@@ -401,7 +401,7 @@ impl CrateListingV2 {
         pkg: &Package,
         bins: &BTreeSet<String>,
         version_req: Option<String>,
-        opts: &CompileOptions<'_>,
+        opts: &CompileOptions,
         target: &str,
         rustc: &str,
     ) {
@@ -424,9 +424,9 @@ impl CrateListingV2 {
         if let Some(info) = self.installs.get_mut(&pkg.package_id()) {
             info.bins.append(&mut bins.clone());
             info.version_req = version_req;
-            info.features = feature_set(&opts.features);
-            info.all_features = opts.all_features;
-            info.no_default_features = opts.no_default_features;
+            info.features = feature_set(&opts.cli_features.features);
+            info.all_features = opts.cli_features.all_features;
+            info.no_default_features = !opts.cli_features.uses_default_features;
             info.profile = opts.build_config.requested_profile.to_string();
             info.target = Some(target.to_string());
             info.rustc = Some(rustc.to_string());
@@ -436,9 +436,9 @@ impl CrateListingV2 {
                 InstallInfo {
                     version_req,
                     bins: bins.clone(),
-                    features: feature_set(&opts.features),
-                    all_features: opts.all_features,
-                    no_default_features: opts.no_default_features,
+                    features: feature_set(&opts.cli_features.features),
+                    all_features: opts.cli_features.all_features,
+                    no_default_features: !opts.cli_features.uses_default_features,
                     profile: opts.build_config.requested_profile.to_string(),
                     target: Some(target.to_string()),
                     rustc: Some(rustc.to_string()),
@@ -490,17 +490,12 @@ impl InstallInfo {
     /// Determine if this installation is "up to date", or if it needs to be reinstalled.
     ///
     /// This does not do Package/Source/Version checking.
-    fn is_up_to_date(
-        &self,
-        opts: &CompileOptions<'_>,
-        target: &str,
-        exes: &BTreeSet<String>,
-    ) -> bool {
-        self.features == feature_set(&opts.features)
-            && self.all_features == opts.all_features
-            && self.no_default_features == opts.no_default_features
+    fn is_up_to_date(&self, opts: &CompileOptions, target: &str, exes: &BTreeSet<String>) -> bool {
+        self.features == feature_set(&opts.cli_features.features)
+            && self.all_features == opts.cli_features.all_features
+            && self.no_default_features != opts.cli_features.uses_default_features
             && self.profile.as_str() == opts.build_config.requested_profile.as_str()
-            && (self.target.is_none() || self.target.as_ref().map(|t| t.as_ref()) == Some(target))
+            && (self.target.is_none() || self.target.as_deref() == Some(target))
             && &self.bins == exes
     }
 }
@@ -526,16 +521,14 @@ pub fn path_source(source_id: SourceId, config: &Config) -> CargoResult<PathSour
 }
 
 /// Gets a Package based on command-line requirements.
-pub fn select_pkg<'a, T>(
-    mut source: T,
-    name: Option<&str>,
-    vers: Option<&str>,
+pub fn select_dep_pkg<T>(
+    source: &mut T,
+    dep: Dependency,
     config: &Config,
     needs_update: bool,
-    list_all: &mut dyn FnMut(&mut T) -> CargoResult<Vec<Package>>,
 ) -> CargoResult<Package>
 where
-    T: Source + 'a,
+    T: Source,
 {
     // This operation may involve updating some sources or making a few queries
     // which may involve frobbing caches, as a result make sure we synchronize
@@ -543,86 +536,79 @@ where
     let _lock = config.acquire_package_cache_lock()?;
 
     if needs_update {
-        source.update()?;
+        source.invalidate_cache();
     }
 
-    if let Some(name) = name {
-        let vers = if let Some(v) = vers {
-            // If the version begins with character <, >, =, ^, ~ parse it as a
-            // version range, otherwise parse it as a specific version
-            let first = v
-                .chars()
-                .next()
-                .ok_or_else(|| format_err!("no version provided for the `--vers` flag"))?;
-
-            let is_req = "<>=^~".contains(first) || v.contains('*');
-            if is_req {
-                match v.parse::<VersionReq>() {
-                    Ok(v) => Some(v.to_string()),
-                    Err(_) => bail!(
-                        "the `--vers` provided, `{}`, is \
-                         not a valid semver version requirement\n\n\
-                         Please have a look at \
-                         https://doc.rust-lang.org/cargo/reference/specifying-dependencies.html \
-                         for the correct format",
-                        v
-                    ),
+    let deps = loop {
+        match source.query_vec(&dep, QueryKind::Exact)? {
+            Poll::Ready(deps) => break deps,
+            Poll::Pending => source.block_until_ready()?,
+        }
+    };
+    match deps.iter().map(|p| p.package_id()).max() {
+        Some(pkgid) => {
+            let pkg = Box::new(source).download_now(pkgid, config)?;
+            Ok(pkg)
+        }
+        None => {
+            let is_yanked: bool = if dep.version_req().is_exact() {
+                let version: String = dep.version_req().to_string();
+                if let Ok(pkg_id) =
+                    PackageId::new(dep.package_name(), &version[1..], source.source_id())
+                {
+                    source.invalidate_cache();
+                    loop {
+                        match source.is_yanked(pkg_id) {
+                            Poll::Ready(Ok(is_yanked)) => break is_yanked,
+                            Poll::Ready(Err(_)) => break false,
+                            Poll::Pending => source.block_until_ready()?,
+                        }
+                    }
+                } else {
+                    false
                 }
             } else {
-                match v.to_semver() {
-                    Ok(v) => Some(format!("={}", v)),
-                    Err(e) => {
-                        let mut msg = format!(
-                            "the `--vers` provided, `{}`, is \
-                             not a valid semver version: {}\n",
-                            v, e
-                        );
-
-                        // If it is not a valid version but it is a valid version
-                        // requirement, add a note to the warning
-                        if v.parse::<VersionReq>().is_ok() {
-                            msg.push_str(&format!(
-                                "\nif you want to specify semver range, \
-                                 add an explicit qualifier, like ^{}",
-                                v
-                            ));
-                        }
-                        bail!(msg);
-                    }
-                }
-            }
-        } else {
-            None
-        };
-        let vers = vers.as_ref().map(|s| &**s);
-        let vers_spec = if vers.is_none() && source.source_id().is_registry() {
-            // Avoid pre-release versions from crate.io
-            // unless explicitly asked for
-            Some("*")
-        } else {
-            vers
-        };
-        let dep = Dependency::parse_no_deprecated(name, vers_spec, source.source_id())?;
-        let deps = source.query_vec(&dep)?;
-        match deps.iter().map(|p| p.package_id()).max() {
-            Some(pkgid) => {
-                let pkg = Box::new(&mut source).download_now(pkgid, config)?;
-                Ok(pkg)
-            }
-            None => {
-                let vers_info = vers
-                    .map(|v| format!(" with version `{}`", v))
-                    .unwrap_or_default();
+                false
+            };
+            if is_yanked {
                 bail!(
-                    "could not find `{}` in {}{}",
-                    name,
+                    "cannot install package `{}`, it has been yanked from {}",
+                    dep.package_name(),
+                    source.source_id()
+                )
+            } else {
+                bail!(
+                    "could not find `{}` in {} with version `{}`",
+                    dep.package_name(),
                     source.source_id(),
-                    vers_info
+                    dep.version_req(),
                 )
             }
         }
+    }
+}
+
+pub fn select_pkg<T, F>(
+    source: &mut T,
+    dep: Option<Dependency>,
+    mut list_all: F,
+    config: &Config,
+) -> CargoResult<Package>
+where
+    T: Source,
+    F: FnMut(&mut T) -> CargoResult<Vec<Package>>,
+{
+    // This operation may involve updating some sources or making a few queries
+    // which may involve frobbing caches, as a result make sure we synchronize
+    // with other global Cargos
+    let _lock = config.acquire_package_cache_lock()?;
+
+    source.invalidate_cache();
+
+    return if let Some(dep) = dep {
+        select_dep_pkg(source, dep, config, false)
     } else {
-        let candidates = list_all(&mut source)?;
+        let candidates = list_all(source)?;
         let binaries = candidates
             .iter()
             .filter(|cand| cand.targets().iter().filter(|t| t.is_bin()).count() > 0);
@@ -635,23 +621,25 @@ where
                 Some(p) => p,
                 None => bail!(
                     "no packages found with binaries or \
-                     examples"
+                         examples"
                 ),
             },
         };
-        return Ok(pkg.clone());
+        Ok(pkg.clone())
+    };
 
-        fn multi_err(kind: &str, mut pkgs: Vec<&Package>) -> String {
-            pkgs.sort_unstable_by_key(|a| a.name());
-            format!(
-                "multiple packages with {} found: {}",
-                kind,
-                pkgs.iter()
-                    .map(|p| p.name().as_str())
-                    .collect::<Vec<_>>()
-                    .join(", ")
-            )
-        }
+    fn multi_err(kind: &str, mut pkgs: Vec<&Package>) -> String {
+        pkgs.sort_unstable_by_key(|a| a.name());
+        format!(
+            "multiple packages with {} found: {}. When installing a git repository, \
+            cargo will always search the entire repo for any Cargo.toml. \
+            Please specify which to install.",
+            kind,
+            pkgs.iter()
+                .map(|p| p.name().as_str())
+                .collect::<Vec<_>>()
+                .join(", ")
+        )
     }
 }
 
@@ -674,9 +662,9 @@ where
     }
 }
 
-/// Helper to convert features Vec to a BTreeSet.
-fn feature_set(features: &[String]) -> BTreeSet<String> {
-    features.iter().cloned().collect()
+/// Helper to convert features to a BTreeSet.
+fn feature_set(features: &Rc<BTreeSet<FeatureValue>>) -> BTreeSet<String> {
+    features.iter().map(|s| s.to_string()).collect()
 }
 
 /// Helper to get the executable names from a filter.
